@@ -20,19 +20,20 @@
  *
  */
 
+/* \summary: Linux Classical IP over ATM printer */
+
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
 #include <string.h>
 
-#include <netdissect-stdinc.h>
+#include "netdissect-stdinc.h"
 
+#define ND_LONGJMP_FROM_TCHECK
 #include "netdissect.h"
 #include "addrtoname.h"
 
-#define RFC1483LLC_LEN	8
-
 static const unsigned char rfcllc[] = {
 	0xaa,	/* DSAP: non-ISO */
 	0xaa,	/* SSAP: non-ISO */
@@ -41,36 +42,28 @@ static const unsigned char rfcllc[] = {
 	0x00,
 	0x00 };
 
-static inline void
-cip_print(netdissect_options *ndo, int length)
-{
-	/*
-	 * There is no MAC-layer header, so just print the length.
-	 */
-	ND_PRINT((ndo, "%d: ", length));
-}
-
 /*
  * This is the top level routine of the printer.  'p' points
  * to the LLC/SNAP or raw header of the packet, 'h->ts' is the timestamp,
  * 'h->len' is the length of the packet off the wire, and 'h->caplen'
  * is the number of bytes actually captured.
  */
-u_int
+void
 cip_if_print(netdissect_options *ndo, const struct pcap_pkthdr *h, const u_char *p)
 {
 	u_int caplen = h->caplen;
 	u_int length = h->len;
 	int llc_hdrlen;
 
-	if (memcmp(rfcllc, p, sizeof(rfcllc))==0 && caplen < RFC1483LLC_LEN) {
-		ND_PRINT((ndo, "[|cip]"));
-		return (0);
-	}
+	ndo->ndo_protocol = "cip";
 
 	if (ndo->ndo_eflag)
-		cip_print(ndo, length);
+		/*
+		 * There is no MAC-layer header, so just print the length.
+		 */
+		ND_PRINT("%u: ", length);
 
+	ND_TCHECK_LEN(p, sizeof(rfcllc));
 	if (memcmp(rfcllc, p, sizeof(rfcllc)) == 0) {
 		/*
 		 * LLC header is present.  Try to print it & higher layers.
@@ -90,13 +83,5 @@ cip_if_print(netdissect_options *ndo, const struct pcap_pkthdr *h, const u_char
 		ip_print(ndo, p, length);
 	}
 
-	return (llc_hdrlen);
+	ndo->ndo_ll_hdr_len += llc_hdrlen;
 }
-
-
-/*
- * Local Variables:
- * c-style: whitesmith
- * c-basic-offset: 8
- * End:
- */
