@@ -806,6 +806,8 @@ l2tp_print(netdissect_options *ndo, const u_char *dat, u_int length)
 
 	if (flag_o) {	/* Offset Size */
 		pad =  GET_BE_U_2(ptr);
+		/* Offset padding octets in packet buffer? */
+		ND_TCHECK_LEN(ptr + 2, pad);
 		ptr += (2 + pad);
 		cnt += (2 + pad);
 	}
@@ -851,4 +853,7 @@ l2tp_print(netdissect_options *ndo, const u_char *dat, u_int length)
 		ppp_print(ndo, ptr, length - cnt);
 		ND_PRINT("}");
 	}
+	return;
+trunc:
+	nd_print_trunc(ndo);
 }
